@@ -1,40 +1,12 @@
 # coding: utf-8
-# /*##########################################################################
-#
-# Copyright (c) 2016-2022 European Synchrotron Radiation Facility
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
-#
-# ###########################################################################*/
 """module defining utils for an .vol volume (also know as raw)"""
 
-
-__authors__ = ["H. Payno", "P. Paleo"]
-__license__ = "MIT"
-__date__ = "10/01/2023"
-
+from __future__ import annotations
 
 import logging
 import os
 import sys
-from typing import Optional
-from xml.dom.minidom import parseString as parse_xml_string
+from defusedxml.minidom import parseString as parse_xml_string
 
 import h5py
 import numpy
@@ -49,6 +21,8 @@ from tomoscan.volumebase import VolumeBase
 
 _logger = logging.getLogger(__name__)
 
+__all__ = ["RawVolume"]
+
 
 class RawVolume(VolumeBase):
     """
@@ -67,12 +41,12 @@ class RawVolume(VolumeBase):
 
     def __init__(
         self,
-        file_path: Optional[str] = None,
-        data: Optional[numpy.ndarray] = None,
-        source_scan: Optional[TomoScanBase] = None,
-        metadata: Optional[dict] = None,
-        data_url: Optional[DataUrl] = None,
-        metadata_url: Optional[DataUrl] = None,
+        file_path: str | None = None,
+        data: numpy.ndarray | None = None,
+        source_scan: TomoScanBase | None = None,
+        metadata: dict | None = None,
+        data_url: DataUrl | None = None,
+        metadata_url: DataUrl | None = None,
         overwrite: bool = False,
         append: bool = False,
         data_extension=DEFAULT_DATA_EXTENSION,
@@ -80,6 +54,8 @@ class RawVolume(VolumeBase):
     ) -> None:
         if file_path is not None:
             url = DataUrl(file_path=file_path, data_path=None, scheme="raw")
+        else:
+            url = None
 
         self._file_path = file_path
         super().__init__(
@@ -96,14 +72,19 @@ class RawVolume(VolumeBase):
         self.append = append
 
     @property
-    def data_extension(self):
+    def data_extension(self) -> str | None:
+        print(self.data_url.file_path())
         if self.data_url is not None and self.data_url.file_path() is not None:
-            return os.path.splitext(self.data_url.file_path())[1]
+            _, ext = os.path.splitext(self.data_url.file_path())
+            if ext:
+                return ext[1:]
 
     @property
     def metadata_extension(self):
         if self.metadata_url is not None and self.metadata_url.file_path() is not None:
-            return os.path.splitext(self.metadata_url.file_path())[1]
+            _, ext = os.path.splitext(self.metadata_url.file_path())
+            if ext:
+                return ext[1:]
 
     @VolumeBase.data.setter
     def data(self, data):
@@ -120,14 +101,14 @@ class RawVolume(VolumeBase):
         return self._file_path
 
     @file_path.setter
-    def file_path(self, file_path: Optional[str]):
+    def file_path(self, file_path: str | None):
         if not (file_path is None or isinstance(file_path, str)):
             raise TypeError
         self._file_path = file_path
         self.url = DataUrl(file_path=file_path, data_path=None, scheme="raw")
 
     @docstring(VolumeBase)
-    def deduce_data_and_metadata_urls(self, url: Optional[DataUrl]) -> tuple:
+    def deduce_data_and_metadata_urls(self, url: DataUrl | None) -> tuple:
         if url is None:
             return None, None
         else:
@@ -155,7 +136,7 @@ class RawVolume(VolumeBase):
             )
 
     @docstring(VolumeBase)
-    def save_data(self, url: Optional[DataUrl] = None, **kwargs) -> None:
+    def save_data(self, url: DataUrl | None = None, **kwargs) -> None:
         if self.data is None:
             return
         url = url or self.data_url
@@ -196,9 +177,7 @@ class RawVolume(VolumeBase):
             data.tofile(fdesc)
 
     @docstring(VolumeBase)
-    def load_data(
-        self, url: Optional[DataUrl] = None, store: bool = True
-    ) -> numpy.array:
+    def load_data(self, url: DataUrl | None = None, store: bool = True) -> numpy.array:
         url = url or self.data_url
         if url is None:
             raise ValueError(
@@ -244,7 +223,7 @@ class RawVolume(VolumeBase):
         return data
 
     @docstring(VolumeBase)
-    def save_metadata(self, url: Optional[DataUrl] = None, store: bool = True) -> None:
+    def save_metadata(self, url: DataUrl | None = None, store: bool = True) -> None:
         """
         :raises KeyError: if data path already exists and overwrite set to False
         :raises ValueError: if data is None
@@ -317,7 +296,7 @@ class RawVolume(VolumeBase):
             raise ValueError(f"scheme {url.scheme()} is not handled")
 
     @docstring(VolumeBase)
-    def load_metadata(self, url: Optional[DataUrl] = None, store: bool = True) -> dict:
+    def load_metadata(self, url: DataUrl | None = None, store: bool = True) -> dict:
         url = url or self.metadata_url
         if url is None:
             raise ValueError(
