@@ -1,37 +1,9 @@
 # coding: utf-8
-# /*##########################################################################
-#
-# Copyright (c) 2016-2022 European Synchrotron Radiation Facility
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
-#
-# ###########################################################################*/
 """module defining utils for a tiff volume"""
 
-
-__authors__ = ["H. Payno", "P. Paleo"]
-__license__ = "MIT"
-__date__ = "01/02/2022"
-
+from __future__ import annotations
 
 import os
-from typing import Optional
 
 import numpy
 from silx.io.dictdump import dicttoini
@@ -42,10 +14,13 @@ from tomoscan.esrf.identifier.tiffidentifier import (
     MultiTiffVolumeIdentifier,
     TIFFVolumeIdentifier,
 )
-from tomoscan.esrf.volume.singleframebase import VolumeSingleFrameBase
+from tomoscan.esrf.volume.singleframebase import (
+    VolumeSingleFrameBase,
+    TIFFLikeDiskAccessor,
+)
 from tomoscan.scanbase import TomoScanBase
 from tomoscan.utils import docstring, get_subvolume_shape
-from tomoscan.volumebase import VolumeBase
+from tomoscan.volumebase import VolumeBase, SliceTuple
 
 try:
     import tifffile  # noqa #F401 needed for later possible lazy loading
@@ -61,6 +36,9 @@ import logging
 _logger = logging.getLogger(__name__)
 
 
+__all__ = ["check_has_tiffle_file", "TIFFVolume", "MultiTIFFVolume"]
+
+
 def check_has_tiffle_file(handle_mode: str):
     assert handle_mode in ("warning", "raises")
 
@@ -72,7 +50,7 @@ def check_has_tiffle_file(handle_mode: str):
             raise ValueError(message)
 
 
-class TIFFVolume(VolumeSingleFrameBase):
+class TIFFVolume(TIFFLikeDiskAccessor, VolumeSingleFrameBase):
     """
     Save volume data to single frame tiff and metadata to .txt files
 
@@ -85,13 +63,13 @@ class TIFFVolume(VolumeSingleFrameBase):
 
     def __init__(
         self,
-        folder: Optional[str] = None,
-        volume_basename: Optional[str] = None,
-        data: Optional[numpy.ndarray] = None,
-        source_scan: Optional[TomoScanBase] = None,
-        metadata: Optional[dict] = None,
-        data_url: Optional[DataUrl] = None,
-        metadata_url: Optional[DataUrl] = None,
+        folder: str | None = None,
+        volume_basename: str | None = None,
+        data: numpy.ndarray | None = None,
+        source_scan: TomoScanBase | None = None,
+        metadata: dict | None = None,
+        data_url: DataUrl | None = None,
+        metadata_url: DataUrl | None = None,
         overwrite: bool = False,
         start_index=0,
         data_extension=DEFAULT_DATA_EXTENSION,
@@ -104,6 +82,7 @@ class TIFFVolume(VolumeSingleFrameBase):
             )
         else:
             url = None
+        TIFFLikeDiskAccessor.__init__(self)
         super().__init__(
             url=url,
             volume_basename=volume_basename,
@@ -137,6 +116,18 @@ class TIFFVolume(VolumeSingleFrameBase):
         else:
             raise ValueError(f"scheme {scheme} is not handled")
 
+    def _get_slices_from_disk(
+        self, slices: tuple[SliceTuple], url: DataUrl | None = None
+    ) -> dict[SliceTuple, numpy.ndarray]:
+        """
+        read from files a couple of slices along any axis.
+        """
+        check_has_tiffle_file("raises")
+        return super()._get_slices_from_disk(
+            slices=slices,
+            url=url,
+        )
+
     # identifier section
 
     @staticmethod
@@ -169,37 +160,72 @@ class TIFFVolume(VolumeSingleFrameBase):
             ]
         )
 
+    # TIFFLikeAccessor implementation
+
+    def read_n_lines_in_file(self, file_name, line_indices) -> tuple:
+        return tuple(
+            [
+                tifffile.imread(file_name, selection=(slice(None, None), index))
+                for index in line_indices
+            ]
+        )
+
+    def read_n_columns_in_file(self, file_name, column_indices):
+        return tuple(
+            [
+                tifffile.imread(file_name, selection=(index, slice(None, None)))
+                for index in column_indices
+            ]
+        )
+
+    def read_file(self, file_name) -> tuple:
+        return tifffile.imread(file_name)
+
 
 class MultiTIFFVolume(VolumeBase):
     """
     Save tiff into a single tiff file
 
-    :param str file_path: path to the multiframe tiff file
+    :param file_path: path to the multiframe tiff file
     """
 
+    DEFAULT_DATA_EXTENSION = "tiff"
+
+    DEFAULT_METADATA_EXTENSION = "txt"
+
     def __init__(
         self,
-        file_path: Optional[str] = None,
-        data: Optional[numpy.ndarray] = None,
-        source_scan: Optional[TomoScanBase] = None,
-        metadata: Optional[dict] = None,
-        data_url: Optional[DataUrl] = None,
-        metadata_url: Optional[DataUrl] = None,
+        file_path: str | None = None,
+        data: numpy.ndarray | None = None,
+        source_scan: TomoScanBase | None = None,
+        metadata: dict | None = None,
+        data_url: DataUrl | None = None,
+        metadata_url: DataUrl | None = None,
         overwrite: bool = False,
         append: bool = False,
+        data_extension=DEFAULT_DATA_EXTENSION,
+        metadata_extension=DEFAULT_METADATA_EXTENSION,
     ) -> None:
         if file_path is not None:
             url = DataUrl(file_path=file_path)
         else:
             url = None
         super().__init__(
-            url, data, source_scan, metadata, data_url, metadata_url, overwrite
+            url,
+            data,
+            source_scan,
+            metadata,
+            data_url,
+            metadata_url,
+            overwrite,
+            data_extension=data_extension,
+            metadata_extension=metadata_extension,
         )
         check_has_tiffle_file("warning")
         self.append = append
 
     @docstring(VolumeBase)
-    def deduce_data_and_metadata_urls(self, url: Optional[DataUrl]) -> tuple:
+    def deduce_data_and_metadata_urls(self, url: DataUrl | None) -> tuple:
         # convention for tiff multiframe:
         # expect the url to provide a path to a the tiff multiframe file. so data_url will be the same as url
         # and the metadata_url will target a prefix_info.txt file with prefix is the tiff file prefix
@@ -229,12 +255,12 @@ class MultiTIFFVolume(VolumeBase):
             )
 
     @docstring(VolumeBase)
-    def save_data(self, url: Optional[DataUrl] = None) -> None:
+    def save_data(self, url: DataUrl | None = None) -> None:
         """
         :raises KeyError: if data path already exists and overwrite set to False
         :raises ValueError: if data is None
         """
-        # to be discussed. Not sure we should raise an error in this case. Could be usefull but this could also be double edged knife
+        # to be discussed. Not sure we should raise an error in this case. Could be useful but this could also be double edged knife
         if self.data is None:
             raise ValueError("No data to be saved")
         check_has_tiffle_file("raises")
@@ -298,7 +324,7 @@ class MultiTIFFVolume(VolumeBase):
             yield _FrameDumper(data_url, append=self.append if i_frame == 0 else True)
 
     @docstring(VolumeBase)
-    def save_metadata(self, url: Optional[DataUrl] = None) -> None:
+    def save_metadata(self, url: DataUrl | None = None) -> None:
         """
         :raises KeyError: if data path already exists and overwrite set to False
         :raises ValueError: if data is None
@@ -326,7 +352,7 @@ class MultiTIFFVolume(VolumeBase):
 
     @docstring(VolumeBase)
     def load_data(
-        self, url: Optional[DataUrl] = None, store: bool = True
+        self, url: DataUrl | None = None, store: bool = True
     ) -> numpy.ndarray:
         url = url or self.data_url
         if url is None:
@@ -342,7 +368,7 @@ class MultiTIFFVolume(VolumeBase):
         return data
 
     @docstring(VolumeBase)
-    def load_metadata(self, url: Optional[DataUrl] = None, store: bool = True) -> dict:
+    def load_metadata(self, url: DataUrl | None = None, store: bool = True) -> dict:
         url = url or self.metadata_url
         if url is None:
             raise ValueError(
@@ -421,7 +447,7 @@ class MultiTIFFVolume(VolumeBase):
             url = url or self.data_url
             if url is None:
                 raise ValueError(
-                    "No data and data_url know and no url provided. Uanble to browse slices"
+                    "No data and data_url know and no url provided. Unable to browse slices"
                 )
             if url.scheme() == "tifffile":
                 if url.data_path() is not None:
@@ -429,15 +455,15 @@ class MultiTIFFVolume(VolumeBase):
 
                 url = url or self.data_url
                 reader = TiffFile(url.file_path())
-                for serie in reader.series:
-                    data = serie.asarray()
+                for series in reader.series:
+                    data = series.asarray()
                     if data.ndim == 3:
                         for data_slice in data:
                             yield data_slice
                     elif data.ndim == 2:
                         yield data
                     else:
-                        raise ValueError("serie is expected to be 2D or 3D")
+                        raise ValueError("series is expected to be 2D or 3D")
             else:
                 raise ValueError(
                     f"Scheme {url.scheme()} is not handled by multiframe tiff"
@@ -449,7 +475,7 @@ class MultiTIFFVolume(VolumeBase):
         url = url or self.data_url
 
         with tifffile.TiffFile(url.file_path()) as t:
-            shapes = [serie.shape for serie in t.series]
+            shapes = [series.shape for series in t.series]
             # assume that all series have the same dimensions for axis 1 and 2
             vol_shape = (len(t.series), shapes[0][0], shapes[0][1])
         return vol_shape
